@@ -68,6 +68,7 @@
 #undef STATIC_GSSAPIV2
 #undef STATIC_KERBEROS4
 #undef STATIC_LOGIN
+#undef STATIC_OTP
 #undef STATIC_PLAIN
 #undef STATIC_SRP
 #undef STATIC_SASLDB
@@ -88,6 +89,18 @@
 #undef WITH_DES
 #undef WITH_SSL_DES
 
+/* do we have sha1 available? */
+#undef WITH_SHA1
+
+/* should we support srp_setpass */
+#undef DO_SRP_SETPASS
+
+/* do we have OPIE for server-side OTP support? */
+#undef HAVE_OPIE
+
+/* should we support otp_setpass */
+#undef DO_OTP_SETPASS
+
 /* Do we have kerberos for plaintext password checking? */
 #undef HAVE_KRB
 
