@@ -1,6 +1,6 @@
 /* canonusr.c - user canonicalization support
  * Rob Siemborski
- * $Id: canonusr.c,v 1.1.2.19 2001/08/24 23:24:36 rbraun Exp $
+ * $Id: canonusr.c,v 1.2 2001/12/04 02:05:25 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -155,7 +155,7 @@ int _sasl_canon_user(sasl_conn_t *conn,
     }
 
     if(result != SASL_OK) return result;
-
+	
     oparams->user = conn->user_buf;
     oparams->ulen = ulen;
     oparams->authid = conn->authid_buf;
@@ -164,7 +164,8 @@ int _sasl_canon_user(sasl_conn_t *conn,
 #ifndef macintosh
     /* finally, do auxprop lookups (server only) */
     if(sconn) {
-	_sasl_auxprop_lookup(sconn->sparams, 0, oparams->user, oparams->ulen);
+	_sasl_auxprop_lookup(sconn->sparams, 0,
+			     oparams->authid, oparams->alen);
     }
 #endif
 
@@ -267,6 +268,7 @@ static int _canonuser_internal(const sasl_utils_t *utils,
     for(;isspace((int)begin_u[ulen-1]) && ulen > 0; ulen--);
     if(begin_u == &(userin[ulen])) {
 	sasl_FREE(in_buf);
+	utils->seterror(utils->conn, 0, "All-whitespace username.");
 	return SASL_FAIL;
     }
 
@@ -278,6 +280,7 @@ static int _canonuser_internal(const sasl_utils_t *utils,
     for(;isspace((int)begin_a[alen-1]) && alen > 0; alen--);
     if(begin_a == &(authidin[alen])) {
 	sasl_FREE(in_buf);
+	utils->seterror(utils->conn, 0, "All-whitespace authid.");
 	return SASL_FAIL;
     }
 
