@@ -1,7 +1,7 @@
 /* SASL server API implementation
  * Rob Siemborski
  * Tim Martin
- * $Id: checkpw.c,v 1.41.2.23 2001/08/15 20:08:10 rjs3 Exp $
+ * $Id: checkpw.c,v 1.43 2001/12/04 02:05:25 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -226,7 +226,7 @@ int _sasl_auxprop_verify_apop(sasl_conn_t *conn,
 			      const char *response,
 			      const char *user_realm __attribute__((unused)))
 {
-    int ret = SASL_FAIL;
+    int ret = SASL_BADAUTH;
     char *userid = NULL;
     char *realm = NULL;
     unsigned char digest[16];
@@ -237,9 +237,8 @@ int _sasl_auxprop_verify_apop(sasl_conn_t *conn,
     MD5_CTX ctx;
     int i;
 
-    if (!conn || !userstr || !challenge || !response) {
-      return SASL_BADPARAM;
-    }
+    if (!conn || !userstr || !challenge || !response)
+       PARAMERROR(conn)
 
     /* We've done the auxprop lookup already (in our caller) */
     ret = prop_getnames(sconn->sparams->propctx, password_request,
@@ -271,6 +270,8 @@ int _sasl_auxprop_verify_apop(sasl_conn_t *conn,
     }
 
  done:
+    if (ret == SASL_BADAUTH) sasl_seterror(conn, SASL_NOLOG,
+					   "login incorrect");
     if (userid) sasl_FREE(userid);
     if (realm)  sasl_FREE(realm);
 
@@ -278,7 +279,7 @@ int _sasl_auxprop_verify_apop(sasl_conn_t *conn,
 }
 #endif /* DO_SASL_CHECKAPOP */
 
-#ifdef HAVE_PWCHECK
+#if defined(HAVE_PWCHECK) || defined(HAVE_SASLAUTHD)
 /*
  * Keep calling the writev() system call with 'fd', 'iov', and 'iovcnt'
  * until all the data is written out or an error occurs.
@@ -321,7 +322,7 @@ static int retry_writev(int fd, struct iovec *iov, int iovcnt)
 	written += n;
 
 	for (i = 0; i < iovcnt; i++) {
-	    if (iov[i].iov_len > n) {
+	    if (iov[i].iov_len > (unsigned) n) {
 		iov[i].iov_base = (char *)iov[i].iov_base + n;
 		iov[i].iov_len -= n;
 		break;
@@ -334,8 +335,9 @@ static int retry_writev(int fd, struct iovec *iov, int iovcnt)
     }
 }
 
+#endif
 
-
+#ifdef HAVE_PWCHECK
 /* pwcheck daemon-authenticated login */
 static int pwcheck_verify_password(sasl_conn_t *conn,
 				   const char *userid, 
@@ -399,19 +401,46 @@ static int pwcheck_verify_password(sasl_conn_t *conn,
 #endif
 
 #ifdef HAVE_SASLAUTHD
+
+/*
+ * Keep calling the read() system call with 'fd', 'buf', and 'nbyte'
+ * until all the data is read in or an error occurs.
+ */
+int retry_read(int fd, void *buf, unsigned nbyte)
+{
+    int n;
+    int nread = 0;
+
+    if (nbyte == 0) return 0;
+
+    for (;;) {
+	n = read(fd, buf, nbyte);
+	if (n == -1 || n == 0) {
+	    if (errno == EINTR || errno == EAGAIN) continue;
+	    return -1;
+	}
+
+	nread += n;
+
+	if (nread >= (int) nbyte) return nread;
+
+	buf += n;
+	nbyte -= n;
+    }
+}
+
 /* saslauthd-authenticated login */
 static int saslauthd_verify_password(sasl_conn_t *conn,
 				   const char *userid, 
 				   const char *passwd,
-				   const char *service __attribute__((unused)),
-				   const char *user_realm 
-				               __attribute__((unused)))
+				   const char *service,
+				   const char *user_realm)
 {
     static char response[1024];
     int s;
     struct sockaddr_un srvaddr;
-    int r, n;
-    unsigned int start;
+    int r;
+    unsigned short count;
     sasl_getopt_t *getopt;
     void *context;
     char pwpath[sizeof(srvaddr.sun_path)];
@@ -441,54 +470,78 @@ static int saslauthd_verify_password(sasl_conn_t *conn,
 
     r = connect(s, (struct sockaddr *) &srvaddr, sizeof(srvaddr));
     if (r == -1) {
-	sasl_seterror(conn, 0, "cannot connect to pwcheck server");
+	sasl_seterror(conn, 0, "cannot connect to saslauthd server");
 	return SASL_FAIL;
     }
 
+    /*
+     * build request of the form:
+     *
+     * count authid count password count service count realm
+     */
     {
-	int u_len;
-	int p_len;
-	char *msg;
+ 	unsigned short u_len, p_len, s_len, r_len;
+ 	struct iovec iov[8];
+ 
+ 	u_len = htons(strlen(userid));
+ 	p_len = htons(strlen(passwd));
+	s_len = htons(strlen(service));
+	r_len = htons((user_realm ? strlen(user_realm) : 0));
 
-	u_len = strlen(userid) + 1;
-	p_len = strlen(passwd) + 1;
-	msg = sasl_ALLOC(u_len + p_len);
-	if (msg == NULL) {
-	    close(s);
-	    sasl_seterror(conn, 0, "not enough memory");
-	    return SASL_NOMEM;
-	}
-	strcpy(msg, userid);
-	strcpy(msg + u_len, passwd);
+	iov[0].iov_base = &u_len;
+ 	iov[0].iov_len = sizeof(u_len);
+        iov[1].iov_base = (void*) userid;
+	iov[1].iov_len = strlen(userid);
+	iov[2].iov_base = &p_len;
+	iov[2].iov_len = sizeof(p_len);
+	iov[3].iov_base = (void*) passwd;
+	iov[3].iov_len = strlen(passwd);
+	iov[4].iov_base = &s_len;
+	iov[4].iov_len = sizeof(s_len);
+	iov[5].iov_base = (void*) service;
+	iov[5].iov_len = strlen(service);
+	iov[6].iov_base = &r_len;
+        iov[6].iov_len = sizeof(r_len);
+	iov[7].iov_base = user_realm ? (void*) user_realm : "";
+	iov[7].iov_len = user_realm ? strlen(user_realm) : 0;
 
-	while (write(s, msg, u_len + p_len) == -1)
-	    switch (errno) {
-	    case EINTR:
-		continue;
-	    default:
-		sasl_FREE(msg);
-		sasl_seterror(conn,0,"write failed");
-		return SASL_FAIL;
-	    }
-
-	sasl_FREE(msg);
+	if (retry_writev(s, iov, 8) == -1) {
+            sasl_seterror(conn, 0, "write failed");
+  	    return SASL_FAIL;
+  	}
     }
-
-    start = 0;
-    while (start < sizeof(response) - 1) {
-	n = read(s, response + start, sizeof(response) - 1 - start);
-	if (n < 1)
-	    break;
-	start += n;
+  
+    /*
+     * read response of the form:
+     *
+     * count result
+     */
+    if (retry_read(s, &count, sizeof(count)) < (int) sizeof(count)) {
+        sasl_seterror(conn, 0, "size read failed");
+	return SASL_FAIL;
     }
-
+  
+    count = ntohs(count);
+    if (count < 2) { /* MUST have at least "OK" or "NO" */
+	close(s);
+        sasl_seterror(conn, 0, "bad response from saslauthd");
+	return SASL_FAIL;
+    }
+  
+    count = (int)sizeof(response) < count ? sizeof(response) : count;
+    if (retry_read(s, response, count) < count) {
+	close(s);
+        sasl_seterror(conn, 0, "read failed");
+	return SASL_FAIL;
+    }
+  
     close(s);
-
-    if (start > 1 && !strncmp(response, "OK", 2))
-	return SASL_OK;
-
-    response[start] = '\0';
-    sasl_seterror(conn,0,response);
+  
+    if (!strncmp(response, "OK", 2))
+      return SASL_OK;
+  
+    response[count] = '\0';
+    sasl_seterror(conn, 0, "authentication failed");
     return SASL_BADAUTH;
 }
 
