@@ -1,7 +1,7 @@
 /* SASL server API implementation
  * Rob Siemborski
  * Tim Martin
- * $Id: server.c,v 1.84.2.57 2001/08/17 16:49:17 rjs3 Exp $
+ * $Id: server.c,v 1.89 2001/12/04 02:05:27 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -830,6 +830,7 @@ static int mech_permitted(sasl_conn_t *conn,
     const sasl_server_plug_t *plug;
     int myflags;
     context_list_t *cur;
+    sasl_getopt_t *getopt;
     void *context;
     sasl_ssf_t minssf = 0;
 
@@ -842,6 +843,32 @@ static int mech_permitted(sasl_conn_t *conn,
     
     plug = mech->plug;
 
+    /* get the list of allowed mechanisms (default = all) */
+    if (_sasl_getcallback(conn, SASL_CB_GETOPT, &getopt, &context)
+            == SASL_OK) {
+	const char *mlist;
+
+	getopt(context, NULL, "mech_list", &mlist, NULL);
+
+	/* if we have a list, check the plugin against it */
+	if (mlist) {
+	    const char *cp;
+
+	    while (*mlist) {
+		for (cp = mlist; *cp && !isspace((int) *cp); cp++);
+		if (((size_t) (cp - mlist) == strlen(plug->mech_name)) &&
+		    !strncasecmp(mlist, plug->mech_name,
+				 strlen(plug->mech_name))) {
+		    break;
+		}
+		mlist = cp;
+		while (*mlist && isspace((int) *mlist)) mlist++;
+	    }
+
+	    if (!*mlist) return 0;  /* reached EOS -> not in our list */
+	}
+    }
+
     /* setup parameters for the call to mech_avail */
     s_conn->sparams->serverFQDN=conn->serverFQDN;
     s_conn->sparams->service=conn->service;
@@ -976,8 +1003,8 @@ static int do_authorization(sasl_server_conn_t *s_conn)
     }
 
     ret = authproc(&(s_conn->base), auth_context,
-		   s_conn->base.oparams.authid, s_conn->base.oparams.alen,
 		   s_conn->base.oparams.user, s_conn->base.oparams.ulen,
+		   s_conn->base.oparams.authid, s_conn->base.oparams.alen,
 		   s_conn->user_realm,
 		   (s_conn->user_realm ? strlen(s_conn->user_realm) : 0),
 		   s_conn->sparams->propctx);
@@ -1164,6 +1191,16 @@ int sasl_server_start(sasl_conn_t *conn,
     }
 
  done:
+    if(   result != SASL_OK
+       && result != SASL_CONTINUE
+       && result != SASL_INTERACT) {
+	if(conn->context) {
+	    s_conn->mech->plug->mech_dispose(conn->context,
+					     s_conn->sparams->utils);
+	    conn->context = NULL;
+	}
+    }
+    
     RETURN(conn,result);
 }
 
@@ -1231,8 +1268,25 @@ int sasl_server_step(sasl_conn_t *conn,
 	if(!conn->oparams.maxoutbuf) {
 	    conn->oparams.maxoutbuf = conn->props.maxbufsize;
 	}
+
+	/* If authorization id is not the same as authenticationid, get
+	 * the real auxprops */
+	if(strcmp(conn->oparams.user, conn->oparams.authid)) {
+	    prop_clear(s_conn->sparams->propctx,0);
+	    _sasl_auxprop_lookup(s_conn->sparams, 0,
+				 conn->oparams.user, conn->oparams.ulen);
+	}	
     }
     
+    if(   ret != SASL_OK
+       && ret != SASL_CONTINUE
+       && ret != SASL_INTERACT) {
+	if(conn->context) {
+	    s_conn->mech->plug->mech_dispose(conn->context,
+					     s_conn->sparams->utils);
+	    conn->context = NULL;
+	}
+    }
 
     RETURN(conn, ret);
 }
