@@ -1,7 +1,7 @@
 /* CRAM-MD5 SASL plugin
  * Rob Siemborski
  * Tim Martin 
- * $Id: cram.c,v 1.55.2.21 2001/08/03 20:39:33 rjs3 Exp $
+ * $Id: cram.c,v 1.59 2001/12/04 02:06:46 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -52,7 +52,6 @@
 #include <sys/stat.h>
 #endif
 #include <fcntl.h>
-#include <assert.h>
 
 #include <sasl.h>
 #include <saslplug.h>
@@ -118,8 +117,9 @@ static int crammd5_server_mech_new(void *glob_context __attribute__((unused)),
 static void crammd5_both_mech_dispose(void *conn_context,
 				      const sasl_utils_t *utils)
 {
-  context_t *text;
-  text=conn_context;
+  context_t *text=(context_t *)conn_context;
+
+  if(!text) return;
 
   if(text->out_buf) utils->free(text->out_buf);
 
@@ -164,10 +164,10 @@ static int parseuser(const sasl_utils_t *utils,
     int ret;
     char *r;
 
-    assert(user);
-    assert(realm);
-    assert(serverFQDN);
-    assert(input);
+    if(!user || !realm || !serverFQDN || !input) {
+	PARAMERROR( utils );
+	return SASL_BADPARAM;
+    }
 
     if (!user_realm) {
 	ret = _plug_strdup(utils, serverFQDN, realm, NULL);
