@@ -1,7 +1,7 @@
 /* GSSAPI SASL plugin
  * Leif Johansson
  * Rob Siemborski (SASL v2 Conversion)
- * $Id: gssapi.c,v 1.41.2.28 2001/08/20 15:33:23 rjs3 Exp $
+ * $Id: gssapi.c,v 1.43 2001/12/04 02:06:47 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -254,18 +254,26 @@ sasl_gss_encode(void *context, const struct iovec *invec, unsigned numiov,
   gss_buffer_t input_token, output_token;
   gss_buffer_desc real_input_token, real_output_token;
   int ret;
+  struct buffer_info *inblob, bufinfo;
 
   if(!output) return SASL_BADPARAM;
   
-  ret = _plug_iovec_to_buf(text->utils, invec, numiov, &text->enc_in_buf);
-  if(ret != SASL_OK) return ret;
-
+  if(numiov > 1) {
+      ret = _plug_iovec_to_buf(text->utils, invec, numiov, &text->enc_in_buf);
+      if(ret != SASL_OK) return ret;
+      inblob = text->enc_in_buf;
+  } else {
+      bufinfo.data = invec[0].iov_base;
+      bufinfo.curlen = invec[0].iov_len;
+      inblob = &bufinfo;
+  }
+  
   if (text->state != SASL_GSSAPI_STATE_AUTHENTICATED) return SASL_NOTDONE;
   
   input_token = &real_input_token;
   
-  real_input_token.value  = text->enc_in_buf->data;
-  real_input_token.length = text->enc_in_buf->curlen;
+  real_input_token.value  = inblob->data;
+  real_input_token.length = inblob->curlen;
   
   output_token = &real_output_token;
   output_token->value = NULL;
@@ -535,6 +543,8 @@ sasl_gss_free_context_contents(context_t *text)
 {
   OM_uint32 maj_stat, min_stat;
 
+  if (!text) return;
+
   if (text->gss_ctx != GSS_C_NO_CONTEXT) {
     maj_stat = gss_delete_sec_context (&min_stat,&text->gss_ctx,GSS_C_NO_BUFFER);
     text->gss_ctx = GSS_C_NO_CONTEXT;
