@@ -1,7 +1,7 @@
 /* Kerberos4 SASL plugin
  * Rob Siemborski
  * Tim Martin 
- * $Id: kerberos4.c,v 1.65.2.42 2001/08/24 23:26:49 rbraun Exp $
+ * $Id: kerberos4.c,v 1.70 2001/12/04 02:06:48 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -216,25 +216,33 @@ static int kerberosv4_encode(void *context,
 {
   int len, ret;
   context_t *text = (context_t *)context;
+  struct buffer_info *inblob, bufinfo;
 
-  ret = _plug_iovec_to_buf(text->utils, invec, numiov, &text->enc_in_buf);
-  if(ret != SASL_OK) return ret;
-
+  if(numiov > 1) {
+      ret = _plug_iovec_to_buf(text->utils, invec, numiov, &text->enc_in_buf);
+      if(ret != SASL_OK) return ret;
+      inblob = text->enc_in_buf;
+  } else {
+      bufinfo.data = invec[0].iov_base;
+      bufinfo.curlen = invec[0].iov_len;
+      inblob = &bufinfo;
+  }
+  
   ret = _plug_buf_alloc(text->utils, &(text->encode_buf),
-			&text->encode_buf_len, text->enc_in_buf->curlen+40);
+			&text->encode_buf_len, inblob->curlen+40);
 
   if(ret != SASL_OK) return ret;
 
   KRB_LOCK_MUTEX(text->utils);
   
   if(text->sec_type == KRB_SEC_ENCRYPTION) {
-      len=krb_mk_priv(text->enc_in_buf->data, (text->encode_buf+4),
-		      text->enc_in_buf->curlen,  text->init_keysched, 
+      len=krb_mk_priv(inblob->data, (text->encode_buf+4),
+		      inblob->curlen,  text->init_keysched, 
 		      &text->session, &text->ip_local,
 		      &text->ip_remote);
   } else if (text->sec_type == KRB_SEC_INTEGRITY) {
-      len=krb_mk_safe(text->enc_in_buf->data, (text->encode_buf+4),
-		      text->enc_in_buf->curlen,
+      len=krb_mk_safe(inblob->data, (text->encode_buf+4),
+		      inblob->curlen,
 		      &text->session, &text->ip_local, &text->ip_remote);
   } else {
       len = -1;
@@ -459,6 +467,8 @@ static void kerberosv4_both_mech_dispose(void *conn_context,
 {
     context_t *text = (context_t *)conn_context;
 
+    if(!text) return;
+
     if (text->buffer) utils->free(text->buffer);
     if (text->encode_buf) utils->free(text->encode_buf);
     if (text->decode_buf) utils->free(text->decode_buf);
