@@ -1,7 +1,7 @@
 /* SASL server API implementation
  * Rob Siemborski
  * Tim Martin
- * $Id: sasldb.c,v 1.1.2.7 2001/08/06 18:05:39 rjs3 Exp $
+ * $Id: sasldb.c,v 1.2 2001/12/04 02:06:49 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -48,7 +48,6 @@
 /* checkpw stuff */
 
 #include <stdio.h>
-#include <assert.h>
 
 #include "sasl.h"
 #include "saslutil.h"
@@ -65,7 +64,10 @@ static int parseuser(const sasl_utils_t *utils,
     int ret;
     char *r;
 
-    assert(user && serverFQDN);
+    if(!user || !serverFQDN) {
+	PARAMERROR( utils );
+	return SASL_BADPARAM;
+    }
 
     r = strchr(input, '@');
     if (!r) {
