@@ -15,6 +15,9 @@
 /* do we have SIA for plaintext password checking? */
 #undef HAVE_SIA
 
+/* Do we want to enable the experimental sasldb authentication module? */
+#undef AUTH_SASLDB
+
 /* do we have a getuserpw? */
 #undef HAVE_GETUSERPW
 
@@ -37,6 +40,12 @@
 #undef STATIC_GSSAPIV2
 #undef STATIC_KERBEROS4
 #undef STATIC_PLAIN
+#undef HAVE_DB3_DB_H
+#undef SASL_BERKELEYDB
+#undef SASL_DB_PATH
+#undef SASL_GDBM
+#undef SASL_NDBM
+#undef STATIC_SASLDB
 
 @BOTTOM@
 
