@@ -1,7 +1,7 @@
 /* testsuite.c -- Stress the library a little
  * Rob Siemborski
  * Tim Martin
- * $Id: testsuite.c,v 1.13.2.34 2001/08/13 19:15:59 rjs3 Exp $
+ * $Id: testsuite.c,v 1.15 2001/12/04 02:07:02 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -55,7 +55,6 @@
 /*
  * TODO [FIXME]:
  *  put in alloc() routines that fail occasionally.
- *  verify ssf's
  */
 
 #include <config.h>
@@ -63,12 +62,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <sasl/sasl.h>
-#include <sasl/saslutil.h>
-#include <sasl/prop.h>
-#include <sasl/md5global.h>
-#include <sasl/md5.h>
-#include <sasl/hmac-md5.h>
+#include <sasl.h>
+#include <saslutil.h>
+#include <prop.h>
+#include <md5global.h>
+#include <md5.h>
+#include <hmac-md5.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -94,10 +93,13 @@ char myhostname[1024+1];
 const char *username = "rjs3";
 const char *nonexistant_username = "ABCDEFGHIJ";
 const char *authname = "rjs3";
+const char *proxyasname = "rjs3proxy";
 const char *password = "1234";
 const char *cu_plugin = "INTERNAL";
 char other_result[1024];
 
+int proxyflag = 0;
+
 static const char *gssapi_service = "host";
 
 /* our types of failures */
@@ -266,7 +268,7 @@ void test_free(void *ptr)
     free(ptr);
 }
 
-#endif WITH_DMALLOC
+#endif /* WITH_DMALLOC */
 
 int mem_stat() 
 {
@@ -417,6 +419,36 @@ static struct sasl_callback emptysasl_cb[] = {
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
+static int proxy_authproc(sasl_conn_t *conn,
+			   void *context __attribute__((unused)),
+			   const char *requested_user,
+			   unsigned rlen __attribute__((unused)),
+			   const char *auth_identity,
+			   unsigned alen __attribute__((unused)),
+			   const char *def_realm __attribute__((unused)),
+			   unsigned urlen __attribute__((unused)),
+			   struct propctx *propctx __attribute__((unused)))
+{
+    if(!strcmp(auth_identity, authname)
+       && !strcmp(requested_user, proxyasname)) return SASL_OK;
+
+    if(!strcmp(auth_identity, requested_user)) {
+	printf("Warning: Authenticated name but DID NOT proxy (%s/%s)\n",
+	       requested_user, auth_identity);
+	return SASL_OK;
+    }
+
+    sasl_seterror(conn, SASL_NOLOG, "authorization failed: %s by %s",
+		  requested_user, auth_identity);
+    return SASL_BADAUTH;
+}
+
+static struct sasl_callback goodsaslproxy_cb[] = {
+    { SASL_CB_PROXY_POLICY, &proxy_authproc, NULL },
+    { SASL_CB_GETOPT, &good_getopt, NULL },
+    { SASL_CB_LIST_END, NULL, NULL }
+};
+
 char really_long_string[REALLY_LONG_LENGTH];
 
 /*
@@ -845,8 +877,10 @@ void interaction (int id, const char *prompt,
 {
     if (id==SASL_CB_PASS) {
 	*tresult=(char *) password;
-    } else if (id==SASL_CB_USER) {
+    } else if (id==SASL_CB_USER && proxyflag == 0) {
 	*tresult=(char *) username;
+    } else if (id==SASL_CB_USER && proxyflag == 1) {
+	*tresult=(char *) proxyasname;
     } else if (id==SASL_CB_AUTHNAME) {
 	*tresult=(char *) authname;
 #ifdef SASL_CB_GETREALM
@@ -1267,11 +1301,17 @@ int doauth(char *mech, sasl_conn_t **server_conn, sasl_conn_t **client_conn,
 	if(!fail_ok) fatal("Unable to init client");
 	else return result;
     }
-    result=sasl_server_init(goodsasl_cb,"TestSuite");
-    if (result!=SASL_OK) {
-	if(!fail_ok) fatal("unable to init server");
-	return result;
+
+    if(proxyflag == 0) {
+        result = sasl_server_init(goodsasl_cb,"TestSuite");
+    } else {
+        result = sasl_server_init(goodsaslproxy_cb,"TestSuite");
     }
+    if(result != SASL_OK) {
+       if(!fail_ok) fatal("unable to init server");
+       else return result;
+    }
+	    
 
     if ((hp = gethostbyname(myhostname)) == NULL) {
 	perror("gethostbyname");
@@ -1410,9 +1450,12 @@ int doauth_noclientfirst(char *mech, sasl_conn_t **server_conn,
     
     if (strcmp(mech,"GSSAPI")==0) service = gssapi_service;
 
-    if (sasl_client_init(client_callbacks)!=SASL_OK) fatal("Unable to init client");
 
-    if (sasl_server_init(goodsasl_cb,"TestSuite")!=SASL_OK) fatal("unable to init server");
+    if (sasl_client_init(client_callbacks)!=SASL_OK)
+	fatal("Unable to init client");
+
+    if (sasl_server_init(goodsasl_cb,"TestSuite")!=SASL_OK)
+        fatal("unable to init server");  
 
     if ((hp = gethostbyname(myhostname)) == NULL) {
 	perror("gethostbyname");
@@ -1802,6 +1845,24 @@ const sasl_security_properties_t int_only = {
     NULL	    
 };
 
+const sasl_security_properties_t force_des = {
+    0,
+    55,
+    8192,
+    0,
+    NULL,
+    NULL	    
+};
+
+const sasl_security_properties_t force_3des = {
+    0,
+    112,
+    8192,
+    0,
+    NULL,
+    NULL	    
+};
+
 
 const sasl_security_properties_t no_int = {
     2,
@@ -1821,6 +1882,31 @@ const sasl_security_properties_t disable_seclayer = {
     NULL	    
 };
 
+void do_proxypolicy_test(char *mech, void *rock __attribute__((unused))) 
+{
+    sasl_conn_t *sconn, *cconn;
+    const char *username;
+    
+    printf("%s --> start\n", mech);
+    proxyflag = 1;
+    if(doauth(mech, &sconn, &cconn, &security_props, 0) != SASL_OK) {
+	fatal("doauth failed in do_proxypolicy_test");
+    }
+
+    if(sasl_getprop(sconn, SASL_USERNAME, (const void **)&username) != SASL_OK)
+    {
+	fatal("getprop failed in do_proxypolicy_test");
+    }
+    
+    if(strcmp(username, proxyasname)) {
+	printf("Warning: Server Authorization Name != proxyasuser\n");
+    }
+
+    cleanup_auth(&cconn, &sconn);
+    proxyflag = 0;
+    printf("%s --> successful result\n",mech);
+}
+
 void test_clientfirst(char *mech, void *rock __attribute__((unused))) 
 {
     sasl_conn_t *sconn, *cconn;
@@ -1897,12 +1983,14 @@ void testseclayer(char *mech, void *rock __attribute__((unused)))
     const char *txstring = "THIS IS A TEST";
     const char *out, *out2;
     char *tmp;
-    const sasl_security_properties_t *test_props[4] =
+    const sasl_security_properties_t *test_props[6] =
                                           { &security_props,
+					    &force_3des,
+					    &force_des,
 					    &int_only,
 					    &no_int,
 					    &disable_seclayer };
-    const unsigned num_properties = 4;
+    const unsigned num_properties = 6;
     unsigned i;
     const sasl_ssf_t *this_ssf;
     unsigned outlen = 0, outlen2 = 0, totlen = 0;
@@ -2265,6 +2353,11 @@ void test_rand_corrupt(unsigned steps)
     }
 }
 
+void test_proxypolicy() 
+{
+    foreach_mechanism((foreach_t *) &do_proxypolicy_test,NULL);
+}
+
 void test_all_corrupt() 
 {
     tosend_t tosend;
@@ -2639,6 +2732,10 @@ int main(int argc, char **argv)
 	printf("Random tests... skipped\n");
     }
 
+    printf("Testing Proxy Policy...\n");
+    test_proxypolicy();
+    printf("Tests of Proxy Policy...ok\n");
+
     printf("Testing security layer...\n");
     test_seclayer();
     printf("Tests of security layer... ok\n");
